/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ryos_control.h"
#include "roccat_device_sysfs.h"
#include <string.h>

static gchar const * const control_name = "control";

gboolean ryos_select(RoccatDevice *ryos, guint profile_index, guint request, GError **error) {
	RyosControl control;
	gboolean retval;

	g_assert(profile_index < RYOS_PROFILE_NUM);

	control.command = RYOS_COMMAND_CONTROL;
	control.value = profile_index;
	control.request = request;

	retval = roccat_device_sysfs_write(ryos, control_name, (gchar const *)&control, sizeof(RyosControl), error);

	/* FIXME Ignoring inapropriate INVALID or CRITICAL answers as long
	 * as firmware requires it
	 */
	if ((request == 0xb2 || request == 0xe0) && retval == FALSE) {
		g_clear_error(error);
		return TRUE;
	}

	return retval;
}
