/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ryos_profile.h"
#include "roccat_device_sysfs.h"

static gchar const * const profile_name = "profile";

guint ryos_profile_read(RoccatDevice *ryos, GError **error) {
	RyosProfile *profile;
	guint result;

	profile = (RyosProfile *)roccat_device_sysfs_read(ryos, profile_name, sizeof(RyosProfile), error);
	if (*error)
		return 0;

	result = profile->profile_index;
	g_free(profile);
	return result;
}

gboolean ryos_profile_write(RoccatDevice *ryos, guint profile_index, GError **error) {
	RyosProfile profile;

	g_assert(profile_index < RYOS_PROFILE_NUM);

	profile.command = RYOS_COMMAND_PROFILE;
	profile.size = sizeof(RyosProfile);
	profile.profile_index = profile_index;

	return roccat_device_sysfs_write(ryos, profile_name, (gchar const *)&profile, sizeof(RyosProfile), error);
}
